#include "ctype.h"
#include "json_reader.h"

typedef struct reader {
  const char *p;
} reader;

reader *make_reader (const char *stuff) {
  reader *input = malloc (sizeof (struct reader));
  input->p = stuff;
  return input;
}

/* advance input */
reader *next (reader *input) {
  input->p++;
  return input;
}

#define peek(input) (*(input)->p)

void skipws(reader *input) {
  while (isspace (peek(input))) {
    next (input);
  }
}  

/* check for character and advance if found */
bool expect (reader *input, char c) {
  if (peek(input) == c) {
    next (input);
    return true;
  } else {
    return false;
  }
}

bool parse_array (reader *input) {
  return false;
}

bool parse_token (reader *input) {
  while (isalpha (peek(input))) {
    next (input);
  }
  return true;
}

bool parse_number (reader *input) {
  expect (input, '-');
  while (isdigit(peek(input))) {
    next (input);
  }
  return true;
}

bool parse_string (reader *input) {
  expect (input, '"');
  while (peek(input) != '"') {
    next (input);
  }
  return expect (input, '"');
}

bool parse_member (reader *input) {
  skipws(input);
  parse_string (input);
  skipws(input);
  if (!expect (input, ':')) {
    return false;
  }
  return parse_element (input);
}

bool parse_object (reader *input) {
  skipws(input);
  if (expect (input, '}')) {
    return true;
  }
  do {
    bool r = parse_member (input);
  } while (expect (input, ','));
  return expect (input, '}');
}

bool parse_value (reader *input) {
  char c = *input->p;
  if (expect(input, '{')) {
    return parse_object (input);
  } else if (expect(input, '[')) {
    return parse_array (input);
  } else if (peek(input) == '"') {
    return parse_string (input);
  } else if (c == '-' || isdigit (c)) {
    return parse_number (input);
  } else if (isalpha(c)) {
    return parse_token(input);
  } else {
    return false;
  }
}


bool parse_element (reader *input) {
  skipws(input);
  bool r = parse_value (input);
  skipws(input);
  return r;
}

bool parse_json (reader *input) {
  bool r = parse_element (input);
  return (r && *input->p == 0);
}
